﻿/********************* 
 * Screen_Scale Test *
 *********************/

import { core, data, sound, util, visual } from './lib/psychojs-2022.1.3.js';
const { PsychoJS } = core;
const { TrialHandler, MultiStairHandler } = data;
const { Scheduler } = util;
//some handy aliases as in the psychopy scripts;
const { abs, sin, cos, PI: pi, sqrt } = Math;
const { round } = util;


// store info about the experiment session:
let expName = 'screen_scale';  // from the Builder filename that created this script
let expInfo = {'participant': '', 'device type': ["Touch screen", "Keyboard"]};

// Start code blocks for 'Before Experiment'
// init psychoJS:
const psychoJS = new PsychoJS({
  debug: true
});

// open window:
psychoJS.openWindow({
  fullscr: true,
  color: new util.Color([0, 0, 0]),
  units: 'pix',
  waitBlanking: true
});
// schedule the experiment:
psychoJS.schedule(psychoJS.gui.DlgFromDict({
  dictionary: expInfo,
  title: expName
}));

const flowScheduler = new Scheduler(psychoJS);
const dialogCancelScheduler = new Scheduler(psychoJS);
psychoJS.scheduleCondition(function() { return (psychoJS.gui.dialogComponent.button === 'OK'); }, flowScheduler, dialogCancelScheduler);

// flowScheduler gets run if the participants presses OK
flowScheduler.add(updateInfo); // add timeStamp
flowScheduler.add(experimentInit);
flowScheduler.add(scale_set_upRoutineBegin());
flowScheduler.add(scale_set_upRoutineEachFrame());
flowScheduler.add(scale_set_upRoutineEnd());
const key_scaleLoopScheduler = new Scheduler(psychoJS);
flowScheduler.add(key_scaleLoopBegin(key_scaleLoopScheduler));
flowScheduler.add(key_scaleLoopScheduler);
flowScheduler.add(key_scaleLoopEnd);
const touch_trialsLoopScheduler = new Scheduler(psychoJS);
flowScheduler.add(touch_trialsLoopBegin(touch_trialsLoopScheduler));
flowScheduler.add(touch_trialsLoopScheduler);
flowScheduler.add(touch_trialsLoopEnd);
flowScheduler.add(quitPsychoJS, '', true);

// quit if user presses Cancel in dialog box:
dialogCancelScheduler.add(quitPsychoJS, '', false);

psychoJS.start({
  expName: expName,
  expInfo: expInfo,
  resources: [
    {'name': 'bank-1300155_640.png', 'path': 'bank-1300155_640.png'},
    {'name': 'continue.png', 'path': 'continue.png'}
  ]
});

psychoJS.experimentLogger.setLevel(core.Logger.ServerLevel.EXP);


var frameDur;
async function updateInfo() {
  expInfo['date'] = util.MonotonicClock.getDateStr();  // add a simple timestamp
  expInfo['expName'] = expName;
  expInfo['psychopyVersion'] = '2022.1.3';
  expInfo['OS'] = window.navigator.platform;

  psychoJS.experiment.dataFileName = (("." + "/") + `data/${expInfo["participant"]}_${expName}_${expInfo["date"]}`);

  // store frame rate of monitor if we can measure it successfully
  expInfo['frameRate'] = psychoJS.window.getActualFrameRate();
  if (typeof expInfo['frameRate'] !== 'undefined')
    frameDur = 1.0 / Math.round(expInfo['frameRate']);
  else
    frameDur = 1.0 / 60.0; // couldn't get a reliable measure so guess

  // add info from the URL:
  util.addInfoFromUrl(expInfo);
  
  return Scheduler.Event.NEXT;
}


var scale_set_upClock;
var show_keys;
var show_touch;
var oldt;
var x_size;
var y_size;
var screen_height;
var x_scale;
var y_scale;
var dbase;
var unittext;
var vsize;
var screen_scale_keysClock;
var text_top;
var text_bottom;
var ccimage;
var rectangle_keysClock;
var rectangle_text_keys;
var polygon_keys;
var key_resp_keys;
var screen_scale_touchClock;
var text_top_touch;
var tri_up;
var tri_down;
var tri_right;
var tri_left;
var ccimage_touch;
var continue_button;
var mouse;
var text_bottom_touch;
var rectangle_touchClock;
var rectangle_text_touch;
var polygon;
var mouse_end;
var globalClock;
var routineTimer;
async function experimentInit() {
  // Initialize components for Routine "scale_set_up"
  scale_set_upClock = new util.Clock();
  show_keys = 0;
  show_touch = 0;
  if ((expInfo["device type"] === "Touch screen")) {
      show_touch = 1;
  } else {
      show_keys = 1;
  }
  oldt = 0;
  x_size = 8.56;
  y_size = 5.398;
  screen_height = 0;
  if ((psychoJS.window.units === "norm")) {
      x_scale = 0.05;
      y_scale = 0.1;
      dbase = 0.0001;
      unittext = " norm units";
      vsize = 2;
  } else {
      if ((psychoJS.window.units === "pix")) {
          x_scale = 60;
          y_scale = 40;
          dbase = 0.1;
          unittext = " pixels";
          vsize = psychoJS.window.size[1];
      } else {
          x_scale = 0.05;
          y_scale = 0.05;
          dbase = 0.0001;
          unittext = " height units";
          vsize = 1;
      }
  }
  
  // Initialize components for Routine "screen_scale_keys"
  screen_scale_keysClock = new util.Clock();
  text_top = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_top',
    text: 'Resize this image to match the size of a credit card\nUp arrow for taller\nDown arrow for shorter\nLeft arrow for narrower\nRight arrow for wider',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0.7], height: 0.1,  wrapWidth: 1.5, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: -1.0 
  });
  
  text_bottom = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_bottom',
    text: 'Press the space bar when done',
    font: 'Arial',
    units: 'norm', 
    pos: [0, (- 0.6)], height: 0.1,  wrapWidth: 1.5, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: -2.0 
  });
  
  ccimage = new visual.ImageStim({
    win : psychoJS.window,
    name : 'ccimage', units : undefined, 
    image : 'bank-1300155_640.png', mask : undefined,
    ori : 0, pos : [0, 0], size : [(x_size * x_scale), (y_size * y_scale)],
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -3.0 
  });
  // Initialize components for Routine "rectangle_keys"
  rectangle_keysClock = new util.Clock();
  rectangle_text_keys = new visual.TextStim({
    win: psychoJS.window,
    name: 'rectangle_text_keys',
    text: 'This shape should be a 10 cm square.\nComponent size  (10*x_scale, 10*y_scale) set every repeat.\nPress space to continue.',
    font: 'Arial',
    units: 'norm', 
    pos: [0, (- 0.8)], height: 0.1,  wrapWidth: 1.5, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: 0.0 
  });
  
  polygon_keys = new visual.Rect ({
    win: psychoJS.window, name: 'polygon_keys', 
    width: [1.0, 1.0][0], height: [1.0, 1.0][1],
    ori: 0, pos: [0, 0],
    lineWidth: 1, lineColor: new util.Color([1, 1, 1]),
    fillColor: new util.Color([1, 1, 1]),
    opacity: 1, depth: -1, interpolate: true,
  });
  
  key_resp_keys = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "screen_scale_touch"
  screen_scale_touchClock = new util.Clock();
  text_top_touch = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_top_touch',
    text: 'Resize this image to match the size of a credit card by clicking/tapping the arrows:\nUp = taller\nDown = shorter\nLeft = narrower\nRight = wider',
    font: 'Arial',
    units: 'norm', 
    pos: [0, 0.7], height: 0.1,  wrapWidth: 1.5, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: -1.0 
  });
  
  tri_up = new visual.ShapeStim ({
    win: psychoJS.window, name: 'tri_up', units : 'height', 
    vertices: [[-[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [+[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [0, [0.1, 0.1][1]/2.0]],
    ori: 0.0, pos: [(- 0.55), 0.35],
    lineWidth: 1.0, lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: 1.0, depth: -2, interpolate: true,
  });
  
  tri_down = new visual.ShapeStim ({
    win: psychoJS.window, name: 'tri_down', units : 'height', 
    vertices: [[-[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [+[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [0, [0.1, 0.1][1]/2.0]],
    ori: 180.0, pos: [(- 0.55), 0.2],
    lineWidth: 1.0, lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: 1.0, depth: -3, interpolate: true,
  });
  
  tri_right = new visual.ShapeStim ({
    win: psychoJS.window, name: 'tri_right', units : 'height', 
    vertices: [[-[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [+[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [0, [0.1, 0.1][1]/2.0]],
    ori: 90.0, pos: [0.65, 0.3],
    lineWidth: 1.0, lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: 1.0, depth: -4, interpolate: true,
  });
  
  tri_left = new visual.ShapeStim ({
    win: psychoJS.window, name: 'tri_left', units : 'height', 
    vertices: [[-[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [+[0.1, 0.1][0]/2.0, -[0.1, 0.1][1]/2.0], [0, [0.1, 0.1][1]/2.0]],
    ori: 270.0, pos: [0.5, 0.3],
    lineWidth: 1.0, lineColor: new util.Color('white'),
    fillColor: new util.Color('white'),
    opacity: 1.0, depth: -5, interpolate: true,
  });
  
  ccimage_touch = new visual.ImageStim({
    win : psychoJS.window,
    name : 'ccimage_touch', units : undefined, 
    image : 'bank-1300155_640.png', mask : undefined,
    ori : 0, pos : [0, 0], size : 1.0,
    color : new util.Color([1, 1, 1]), opacity : 1,
    flipHoriz : false, flipVert : false,
    texRes : 128, interpolate : true, depth : -6.0 
  });
  continue_button = new visual.ImageStim({
    win : psychoJS.window,
    name : 'continue_button', units : 'height', 
    image : 'continue.png', mask : undefined,
    ori : 0.0, pos : [0.65, (- 0.45)], size : [0.25, 0.07],
    color : new util.Color([1,1,1]), opacity : 1.0,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -7.0 
  });
  mouse = new core.Mouse({
    win: psychoJS.window,
  });
  mouse.mouseClock = new util.Clock();
  text_bottom_touch = new visual.TextStim({
    win: psychoJS.window,
    name: 'text_bottom_touch',
    text: '',
    font: 'Arial',
    units: 'norm', 
    pos: [0, (- 0.5)], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: -9.0 
  });
  
  // Initialize components for Routine "rectangle_touch"
  rectangle_touchClock = new util.Clock();
  rectangle_text_touch = new visual.TextStim({
    win: psychoJS.window,
    name: 'rectangle_text_touch',
    text: 'This shape should be a 10 cm square.\nComponent size  (10*x_scale, 10*y_scale) set every repeat.\nClick or touch here to continue.',
    font: 'Arial',
    units: 'norm', 
    pos: [0, (- 0.8)], height: 0.1,  wrapWidth: 1.5, ori: 0,
    color: new util.Color('white'),  opacity: 1,
    depth: 0.0 
  });
  
  polygon = new visual.Rect ({
    win: psychoJS.window, name: 'polygon', 
    width: [1.0, 1.0][0], height: [1.0, 1.0][1],
    ori: 0, pos: [0, 0],
    lineWidth: 1, lineColor: new util.Color([1, 1, 1]),
    fillColor: new util.Color([1, 1, 1]),
    opacity: 1, depth: -1, interpolate: true,
  });
  
  mouse_end = new core.Mouse({
    win: psychoJS.window,
  });
  mouse_end.mouseClock = new util.Clock();
  // Create some handy timers
  globalClock = new util.Clock();  // to track the time since experiment started
  routineTimer = new util.CountdownTimer();  // to track time remaining of each (non-slip) routine
  
  return Scheduler.Event.NEXT;
}


var t;
var frameN;
var continueRoutine;
var scale_set_upComponents;
function scale_set_upRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'scale_set_up'-------
    t = 0;
    scale_set_upClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    scale_set_upComponents = [];
    
    for (const thisComponent of scale_set_upComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function scale_set_upRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'scale_set_up'-------
    // get current time
    t = scale_set_upClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of scale_set_upComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function scale_set_upRoutineEnd() {
  return async function () {
    //------Ending Routine 'scale_set_up'-------
    for (const thisComponent of scale_set_upComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "scale_set_up" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var key_scale;
var currentLoop;
function key_scaleLoopBegin(key_scaleLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    key_scale = new TrialHandler({
      psychoJS: psychoJS,
      nReps: show_keys, method: TrialHandler.Method.RANDOM,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'key_scale'
    });
    psychoJS.experiment.addLoop(key_scale); // add the loop to the experiment
    currentLoop = key_scale;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisKey_scale of key_scale) {
      const snapshot = key_scale.getSnapshot();
      key_scaleLoopScheduler.add(importConditions(snapshot));
      key_scaleLoopScheduler.add(screen_scale_keysRoutineBegin(snapshot));
      key_scaleLoopScheduler.add(screen_scale_keysRoutineEachFrame());
      key_scaleLoopScheduler.add(screen_scale_keysRoutineEnd());
      key_scaleLoopScheduler.add(rectangle_keysRoutineBegin(snapshot));
      key_scaleLoopScheduler.add(rectangle_keysRoutineEachFrame());
      key_scaleLoopScheduler.add(rectangle_keysRoutineEnd());
      key_scaleLoopScheduler.add(endLoopIteration(key_scaleLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


async function key_scaleLoopEnd() {
  psychoJS.experiment.removeLoop(key_scale);

  return Scheduler.Event.NEXT;
}


var touch_trials;
function touch_trialsLoopBegin(touch_trialsLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    touch_trials = new TrialHandler({
      psychoJS: psychoJS,
      nReps: show_touch, method: TrialHandler.Method.RANDOM,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'touch_trials'
    });
    psychoJS.experiment.addLoop(touch_trials); // add the loop to the experiment
    currentLoop = touch_trials;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisTouch_trial of touch_trials) {
      const snapshot = touch_trials.getSnapshot();
      touch_trialsLoopScheduler.add(importConditions(snapshot));
      touch_trialsLoopScheduler.add(screen_scale_touchRoutineBegin(snapshot));
      touch_trialsLoopScheduler.add(screen_scale_touchRoutineEachFrame());
      touch_trialsLoopScheduler.add(screen_scale_touchRoutineEnd());
      touch_trialsLoopScheduler.add(rectangle_touchRoutineBegin(snapshot));
      touch_trialsLoopScheduler.add(rectangle_touchRoutineEachFrame());
      touch_trialsLoopScheduler.add(rectangle_touchRoutineEnd());
      touch_trialsLoopScheduler.add(endLoopIteration(touch_trialsLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


async function touch_trialsLoopEnd() {
  psychoJS.experiment.removeLoop(touch_trials);

  return Scheduler.Event.NEXT;
}


var screen_scale_keysComponents;
function screen_scale_keysRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'screen_scale_keys'-------
    t = 0;
    screen_scale_keysClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    console.log("key start");
    psychoJS.eventManager.clearEvents();
    
    // keep track of which components have finished
    screen_scale_keysComponents = [];
    screen_scale_keysComponents.push(text_top);
    screen_scale_keysComponents.push(text_bottom);
    screen_scale_keysComponents.push(ccimage);
    
    for (const thisComponent of screen_scale_keysComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


var _pj;
var keys;
var dscale;
function screen_scale_keysRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'screen_scale_keys'-------
    // get current time
    t = screen_scale_keysClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    var _pj;
    function _pj_snippets(container) {
        function in_es6(left, right) {
            if (((right instanceof Array) || ((typeof right) === "string"))) {
                return (right.indexOf(left) > (- 1));
            } else {
                if (((right instanceof Map) || (right instanceof Set) || (right instanceof WeakMap) || (right instanceof WeakSet))) {
                    return right.has(left);
                } else {
                    return (left in right);
                }
            }
        }
        container["in_es6"] = in_es6;
        return container;
    }
    _pj = {};
    _pj_snippets(_pj);
    keys = psychoJS.eventManager.getKeys();
    if (keys.length) {
        if (((t - oldt) < 0.5)) {
            dscale = (5 * dbase);
            oldt = t;
        } else {
            dscale = dbase;
            oldt = t;
        }
        if ((_pj.in_es6("space", keys) && (t > 1))) {
            continueRoutine = false;
        } else {
            if (_pj.in_es6("up", keys)) {
                y_scale = (Math.round(((y_scale + dscale) * 10000)) / 10000);
            } else {
                if (_pj.in_es6("down", keys)) {
                    y_scale = (Math.round(((y_scale - dscale) * 10000)) / 10000);
                } else {
                    if (_pj.in_es6("left", keys)) {
                        x_scale = (Math.round(((x_scale - dscale) * 10000)) / 10000);
                    } else {
                        if (_pj.in_es6("right", keys)) {
                            x_scale = (Math.round(((x_scale + dscale) * 10000)) / 10000);
                        }
                    }
                }
            }
        }
        screen_height = (Math.round(((vsize * 10) / y_scale)) / 10);
        text_bottom.text = (((((((("X Scale = " + x_scale.toString()) + unittext) + " per cm, Y Scale = ") + y_scale.toString()) + unittext) + " per cm\nScreen height = ") + screen_height.toString()) + " cm\n\nPress the space bar when done");
        ccimage.size = [(x_size * x_scale), (y_size * y_scale)];
    }
    
    
    // *text_top* updates
    if (t >= 0.0 && text_top.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_top.tStart = t;  // (not accounting for frame time here)
      text_top.frameNStart = frameN;  // exact frame index
      
      text_top.setAutoDraw(true);
    }

    
    // *text_bottom* updates
    if (t >= 0.0 && text_bottom.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_bottom.tStart = t;  // (not accounting for frame time here)
      text_bottom.frameNStart = frameN;  // exact frame index
      
      text_bottom.setAutoDraw(true);
    }

    
    // *ccimage* updates
    if (t >= 0.0 && ccimage.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ccimage.tStart = t;  // (not accounting for frame time here)
      ccimage.frameNStart = frameN;  // exact frame index
      
      ccimage.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of screen_scale_keysComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function screen_scale_keysRoutineEnd() {
  return async function () {
    //------Ending Routine 'screen_scale_keys'-------
    for (const thisComponent of screen_scale_keysComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    psychoJS.experiment.addData("X Scale", x_scale);
    psychoJS.experiment.addData("Y Scale", y_scale);
    
    // the Routine "screen_scale_keys" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _key_resp_keys_allKeys;
var rectangle_keysComponents;
function rectangle_keysRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'rectangle_keys'-------
    t = 0;
    rectangle_keysClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    polygon_keys.setSize([(10 * x_scale), (10 * y_scale)]);
    key_resp_keys.keys = undefined;
    key_resp_keys.rt = undefined;
    _key_resp_keys_allKeys = [];
    // keep track of which components have finished
    rectangle_keysComponents = [];
    rectangle_keysComponents.push(rectangle_text_keys);
    rectangle_keysComponents.push(polygon_keys);
    rectangle_keysComponents.push(key_resp_keys);
    
    for (const thisComponent of rectangle_keysComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function rectangle_keysRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'rectangle_keys'-------
    // get current time
    t = rectangle_keysClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *rectangle_text_keys* updates
    if (t >= 0.0 && rectangle_text_keys.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      rectangle_text_keys.tStart = t;  // (not accounting for frame time here)
      rectangle_text_keys.frameNStart = frameN;  // exact frame index
      
      rectangle_text_keys.setAutoDraw(true);
    }

    
    // *polygon_keys* updates
    if (t >= 0.0 && polygon_keys.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      polygon_keys.tStart = t;  // (not accounting for frame time here)
      polygon_keys.frameNStart = frameN;  // exact frame index
      
      polygon_keys.setAutoDraw(true);
    }

    
    // *key_resp_keys* updates
    if (t >= 0.0 && key_resp_keys.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_resp_keys.tStart = t;  // (not accounting for frame time here)
      key_resp_keys.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_resp_keys.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_resp_keys.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_resp_keys.clearEvents(); });
    }

    if (key_resp_keys.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_resp_keys.getKeys({keyList: ['space'], waitRelease: false});
      _key_resp_keys_allKeys = _key_resp_keys_allKeys.concat(theseKeys);
      if (_key_resp_keys_allKeys.length > 0) {
        key_resp_keys.keys = _key_resp_keys_allKeys[_key_resp_keys_allKeys.length - 1].name;  // just the last key pressed
        key_resp_keys.rt = _key_resp_keys_allKeys[_key_resp_keys_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of rectangle_keysComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function rectangle_keysRoutineEnd() {
  return async function () {
    //------Ending Routine 'rectangle_keys'-------
    for (const thisComponent of rectangle_keysComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    key_resp_keys.stop();
    // the Routine "rectangle_keys" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var text_bottom_touch_text;
var gotValidClick;
var screen_scale_touchComponents;
function screen_scale_touchRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'screen_scale_touch'-------
    t = 0;
    screen_scale_touchClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    psychoJS.eventManager.clearEvents();
    text_bottom_touch_text = " ";
    
    // setup some python lists for storing info about the mouse
    // current position of the mouse:
    mouse.x = [];
    mouse.y = [];
    mouse.leftButton = [];
    mouse.midButton = [];
    mouse.rightButton = [];
    mouse.time = [];
    mouse.clicked_name = [];
    gotValidClick = false; // until a click is received
    mouse.mouseClock.reset();
    // keep track of which components have finished
    screen_scale_touchComponents = [];
    screen_scale_touchComponents.push(text_top_touch);
    screen_scale_touchComponents.push(tri_up);
    screen_scale_touchComponents.push(tri_down);
    screen_scale_touchComponents.push(tri_right);
    screen_scale_touchComponents.push(tri_left);
    screen_scale_touchComponents.push(ccimage_touch);
    screen_scale_touchComponents.push(continue_button);
    screen_scale_touchComponents.push(mouse);
    screen_scale_touchComponents.push(text_bottom_touch);
    
    for (const thisComponent of screen_scale_touchComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


var click;
var prevButtonState;
var _mouseButtons;
var _mouseXYs;
function screen_scale_touchRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'screen_scale_touch'-------
    // get current time
    t = screen_scale_touchClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    click = mouse.clicked_name;
    if (click.length) {
        if (((t - oldt) < 0.5)) {
            dscale = (5 * dbase);
            oldt = t;
        } else {
            dscale = dbase;
            oldt = t;
        }
        if ((mouse.isPressedIn(continue_button) && (t > 1))) {
            continueRoutine = false;
        } else {
            if (mouse.isPressedIn(tri_up)) {
                y_scale = (Math.round(((y_scale + dscale) * 10000)) / 10000);
            } else {
                if (mouse.isPressedIn(tri_down)) {
                    y_scale = (Math.round(((y_scale - dscale) * 10000)) / 10000);
                } else {
                    if (mouse.isPressedIn(tri_left)) {
                        x_scale = (Math.round(((x_scale - dscale) * 10000)) / 10000);
                    } else {
                        if (mouse.isPressedIn(tri_right)) {
                            x_scale = (Math.round(((x_scale + dscale) * 10000)) / 10000);
                        }
                    }
                }
            }
        }
        screen_height = (Math.round(((vsize * 10) / y_scale)) / 10);
        text_bottom_touch_text = (((((((("X Scale = " + x_scale.toString()) + unittext) + " per cm, Y Scale = ") + y_scale.toString()) + unittext) + " per cm\nScreen height = ") + screen_height.toString()) + " cm");
        ccimage_touch.size = [(x_size * x_scale), (y_size * y_scale)];
    }
    
    
    // *text_top_touch* updates
    if (t >= 0.0 && text_top_touch.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_top_touch.tStart = t;  // (not accounting for frame time here)
      text_top_touch.frameNStart = frameN;  // exact frame index
      
      text_top_touch.setAutoDraw(true);
    }

    
    // *tri_up* updates
    if (t >= 0.0 && tri_up.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      tri_up.tStart = t;  // (not accounting for frame time here)
      tri_up.frameNStart = frameN;  // exact frame index
      
      tri_up.setAutoDraw(true);
    }

    
    // *tri_down* updates
    if (t >= 0.0 && tri_down.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      tri_down.tStart = t;  // (not accounting for frame time here)
      tri_down.frameNStart = frameN;  // exact frame index
      
      tri_down.setAutoDraw(true);
    }

    
    // *tri_right* updates
    if (t >= 0.0 && tri_right.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      tri_right.tStart = t;  // (not accounting for frame time here)
      tri_right.frameNStart = frameN;  // exact frame index
      
      tri_right.setAutoDraw(true);
    }

    
    // *tri_left* updates
    if (t >= 0.0 && tri_left.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      tri_left.tStart = t;  // (not accounting for frame time here)
      tri_left.frameNStart = frameN;  // exact frame index
      
      tri_left.setAutoDraw(true);
    }

    
    // *ccimage_touch* updates
    if (t >= 0.0 && ccimage_touch.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      ccimage_touch.tStart = t;  // (not accounting for frame time here)
      ccimage_touch.frameNStart = frameN;  // exact frame index
      
      ccimage_touch.setAutoDraw(true);
    }

    
    if (ccimage_touch.status === PsychoJS.Status.STARTED){ // only update if being drawn
      ccimage_touch.setSize([(x_size * x_scale), (y_size * y_scale)], false);
    }
    
    // *continue_button* updates
    if (t >= 0.0 && continue_button.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      continue_button.tStart = t;  // (not accounting for frame time here)
      continue_button.frameNStart = frameN;  // exact frame index
      
      continue_button.setAutoDraw(true);
    }

    // *mouse* updates
    if (t >= 0.0 && mouse.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      mouse.tStart = t;  // (not accounting for frame time here)
      mouse.frameNStart = frameN;  // exact frame index
      
      mouse.status = PsychoJS.Status.STARTED;
      prevButtonState = mouse.getPressed();  // if button is down already this ISN'T a new click
      }
    if (mouse.status === PsychoJS.Status.STARTED) {  // only update if started and not finished!
      _mouseButtons = mouse.getPressed();
      // check if the mouse was inside our 'clickable' objects
      gotValidClick = false;
      for (const obj of [tri_up, tri_down, tri_left, tri_right]) {
        if (obj.contains(mouse)) {
          gotValidClick = true;
          mouse.clicked_name.push(obj.name)
        }
      }
      _mouseXYs = mouse.getPos();
      mouse.x.push(_mouseXYs[0]);
      mouse.y.push(_mouseXYs[1]);
      mouse.leftButton.push(_mouseButtons[0]);
      mouse.midButton.push(_mouseButtons[1]);
      mouse.rightButton.push(_mouseButtons[2]);
      mouse.time.push(mouse.mouseClock.getTime());
    }
    
    // *text_bottom_touch* updates
    if (t >= 0.0 && text_bottom_touch.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      text_bottom_touch.tStart = t;  // (not accounting for frame time here)
      text_bottom_touch.frameNStart = frameN;  // exact frame index
      
      text_bottom_touch.setAutoDraw(true);
    }

    
    if (text_bottom_touch.status === PsychoJS.Status.STARTED){ // only update if being drawn
      text_bottom_touch.setText(text_bottom_touch_text, false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of screen_scale_touchComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function screen_scale_touchRoutineEnd() {
  return async function () {
    //------Ending Routine 'screen_scale_touch'-------
    for (const thisComponent of screen_scale_touchComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    psychoJS.experiment.addData("X Scale", x_scale);
    psychoJS.experiment.addData("Y Scale", y_scale);
    
    // store data for psychoJS.experiment (ExperimentHandler)
    psychoJS.experiment.addData('mouse.x', mouse.x);
    psychoJS.experiment.addData('mouse.y', mouse.y);
    psychoJS.experiment.addData('mouse.leftButton', mouse.leftButton);
    psychoJS.experiment.addData('mouse.midButton', mouse.midButton);
    psychoJS.experiment.addData('mouse.rightButton', mouse.rightButton);
    psychoJS.experiment.addData('mouse.time', mouse.time);
    psychoJS.experiment.addData('mouse.clicked_name', mouse.clicked_name);
    
    // the Routine "screen_scale_touch" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var rectangle_touchComponents;
function rectangle_touchRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'rectangle_touch'-------
    t = 0;
    rectangle_touchClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    polygon.setSize([(10 * x_scale), (10 * y_scale)]);
    // setup some python lists for storing info about the mouse_end
    // current position of the mouse:
    mouse_end.x = [];
    mouse_end.y = [];
    mouse_end.leftButton = [];
    mouse_end.midButton = [];
    mouse_end.rightButton = [];
    mouse_end.time = [];
    mouse_end.clicked_name = [];
    gotValidClick = false; // until a click is received
    // keep track of which components have finished
    rectangle_touchComponents = [];
    rectangle_touchComponents.push(rectangle_text_touch);
    rectangle_touchComponents.push(polygon);
    rectangle_touchComponents.push(mouse_end);
    
    for (const thisComponent of rectangle_touchComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function rectangle_touchRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'rectangle_touch'-------
    // get current time
    t = rectangle_touchClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *rectangle_text_touch* updates
    if (t >= 0.0 && rectangle_text_touch.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      rectangle_text_touch.tStart = t;  // (not accounting for frame time here)
      rectangle_text_touch.frameNStart = frameN;  // exact frame index
      
      rectangle_text_touch.setAutoDraw(true);
    }

    
    // *polygon* updates
    if (t >= 0.0 && polygon.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      polygon.tStart = t;  // (not accounting for frame time here)
      polygon.frameNStart = frameN;  // exact frame index
      
      polygon.setAutoDraw(true);
    }

    // *mouse_end* updates
    if (t >= 0.0 && mouse_end.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      mouse_end.tStart = t;  // (not accounting for frame time here)
      mouse_end.frameNStart = frameN;  // exact frame index
      
      mouse_end.status = PsychoJS.Status.STARTED;
      mouse_end.mouseClock.reset();
      prevButtonState = mouse_end.getPressed();  // if button is down already this ISN'T a new click
      }
    if (mouse_end.status === PsychoJS.Status.STARTED) {  // only update if started and not finished!
      _mouseButtons = mouse_end.getPressed();
      if (!_mouseButtons.every( (e,i,) => (e == prevButtonState[i]) )) { // button state changed?
        prevButtonState = _mouseButtons;
        if (_mouseButtons.reduce( (e, acc) => (e+acc) ) > 0) { // state changed to a new click
          // check if the mouse was inside our 'clickable' objects
          gotValidClick = false;
          for (const obj of [rectangle_text_touch]) {
            if (obj.contains(mouse_end)) {
              gotValidClick = true;
              mouse_end.clicked_name.push(obj.name)
            }
          }
          _mouseXYs = mouse_end.getPos();
          mouse_end.x.push(_mouseXYs[0]);
          mouse_end.y.push(_mouseXYs[1]);
          mouse_end.leftButton.push(_mouseButtons[0]);
          mouse_end.midButton.push(_mouseButtons[1]);
          mouse_end.rightButton.push(_mouseButtons[2]);
          mouse_end.time.push(mouse_end.mouseClock.getTime());
          if (gotValidClick === true) { // abort routine on response
            continueRoutine = false;
          }
        }
      }
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of rectangle_touchComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function rectangle_touchRoutineEnd() {
  return async function () {
    //------Ending Routine 'rectangle_touch'-------
    for (const thisComponent of rectangle_touchComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // store data for psychoJS.experiment (ExperimentHandler)
    if (mouse_end.x) {  psychoJS.experiment.addData('mouse_end.x', mouse_end.x[0])};
    if (mouse_end.y) {  psychoJS.experiment.addData('mouse_end.y', mouse_end.y[0])};
    if (mouse_end.leftButton) {  psychoJS.experiment.addData('mouse_end.leftButton', mouse_end.leftButton[0])};
    if (mouse_end.midButton) {  psychoJS.experiment.addData('mouse_end.midButton', mouse_end.midButton[0])};
    if (mouse_end.rightButton) {  psychoJS.experiment.addData('mouse_end.rightButton', mouse_end.rightButton[0])};
    if (mouse_end.time) {  psychoJS.experiment.addData('mouse_end.time', mouse_end.time[0])};
    if (mouse_end.clicked_name) {  psychoJS.experiment.addData('mouse_end.clicked_name', mouse_end.clicked_name[0])};
    
    // the Routine "rectangle_touch" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


function endLoopIteration(scheduler, snapshot) {
  // ------Prepare for next entry------
  return async function () {
    if (typeof snapshot !== 'undefined') {
      // ------Check if user ended loop early------
      if (snapshot.finished) {
        // Check for and save orphaned data
        if (psychoJS.experiment.isEntryEmpty()) {
          psychoJS.experiment.nextEntry(snapshot);
        }
        scheduler.stop();
      } else {
        const thisTrial = snapshot.getCurrentTrial();
        if (typeof thisTrial === 'undefined' || !('isTrials' in thisTrial) || thisTrial.isTrials) {
          psychoJS.experiment.nextEntry(snapshot);
        }
      }
    return Scheduler.Event.NEXT;
    }
  };
}


function importConditions(currentLoop) {
  return async function () {
    psychoJS.importAttributes(currentLoop.getCurrentTrial());
    return Scheduler.Event.NEXT;
    };
}


async function quitPsychoJS(message, isCompleted) {
  // Check for and save orphaned data
  if (psychoJS.experiment.isEntryEmpty()) {
    psychoJS.experiment.nextEntry();
  }
  
  
  
  
  
  
  psychoJS.window.close();
  psychoJS.quit({message: message, isCompleted: isCompleted});
  
  return Scheduler.Event.QUIT;
}
